#!/usr/bin/env python
# Author: Mark Bryan Yu

import sys
import datetime
import rundb
import os
from PyQt4 import QtGui
from PyQt4 import QtCore

dbfile = sys.path[0] + os.path.sep + 'runninglog.db'

sqlitedb = rundb.RunningDb(dbfile);

def getDateRange(selectedDay):
    dates = []
    
    delta = datetime.timedelta(1)
    

    while selectedDay.weekday() != 6:
        selectedDay = selectedDay - delta
            
    for i in range(7):
        dates.append(selectedDay)
        selectedDay = selectedDay + delta
        
    return dates

class RunningWidget(QtGui.QWidget):
    def __init__(self, parent=None):
        QtGui.QWidget.__init__(self, parent)

        self.cal = QtGui.QCalendarWidget(self)
        self.cal.setGridVisible(True)
        self.cal.setVerticalHeaderFormat(QtGui.QCalendarWidget.NoVerticalHeader)        
        self.connect(self.cal, QtCore.SIGNAL('selectionChanged()'), self.showDetails)


        self.dateLabel = QtGui.QLabel(self)

        self.errorLabel = QtGui.QLabel('Invalid Miles Value', self)
        self.errorLabel.hide()        
        
        self.milesEdit = QtGui.QLineEdit(self)      
        
        self.setButton = QtGui.QPushButton('Set', self)
        self.connect(self.setButton, QtCore.SIGNAL('clicked()'), self.setMilage)
        
        vbox = QtGui.QVBoxLayout()        
        vbox.addWidget(self.cal)
        vbox.addWidget(self.dateLabel)
                
        grid = QtGui.QGridLayout()
        
        self.labels = [QtGui.QLabel('0 miles'), QtGui.QLabel('0 miles'), QtGui.QLabel('0 miles'), QtGui.QLabel('0 miles'), QtGui.QLabel('0 miles')]
        
        grid.addWidget(QtGui.QLabel('Day:'), 0, 0)
        grid.addWidget(self.labels[0], 0, 1)
        
        grid.addWidget(QtGui.QLabel('Week:'), 1, 0)
        grid.addWidget(self.labels[1], 1, 1)
        
        grid.addWidget(QtGui.QLabel('Month:'), 2, 0)
        grid.addWidget(self.labels[2], 2, 1)
        
        grid.addWidget(QtGui.QLabel('Year:'), 3, 0)
        grid.addWidget(self.labels[3], 3, 1)
        
        grid.addWidget(QtGui.QLabel('Lifetime:'), 4, 0)
        grid.addWidget(self.labels[4], 4, 1)
        
        grid.addWidget(QtGui.QLabel('Set Milage:'), 5, 0)
        grid.addWidget(self.milesEdit, 5, 1)        
        grid.addWidget(self.setButton, 5, 2)  
        
        grid.addWidget(self.errorLabel, 6, 1)  
         
        
        vbox.addLayout(grid)
        
        vbox.addStretch(1)
        
        self.setLayout(vbox)
        
        self.showDetails()
        
    def showDetails(self):
        date = self.cal.selectedDate()
        self.dateLabel.setText(str(date.toString()))
        
        pydate = date.toPyDate()    
        
        # day        
        self.labels[0].setText(str(sqlitedb.getMilesPerDay(pydate.year, pydate.month, pydate.day)) + ' miles')
        
        # week

        mile_week = 0
        for day in getDateRange(pydate):
            mile_week = mile_week + sqlitedb.getMilesPerDay(day.year, day.month, day.day)       
            
        self.labels[1].setText(str(mile_week) + ' miles')    
        
        # month                                
        self.labels[2].setText(str(sqlitedb.getMilesPerMonth(day.year, day.month)) + ' miles')    
        
        # year
        self.labels[3].setText(str(sqlitedb.getMilesPerYear(day.year)) + ' miles')   
        
        # lifetime
        self.labels[4].setText(str(sqlitedb.getMilesLifetime()) + ' miles')           
        
    def setMilage(self):
        try:
            miles = float(self.milesEdit.text())
            
            date = self.cal.selectedDate()                
            pydate = date.toPyDate()                
            
            sqlitedb.insertMiles(pydate.year, pydate.month, pydate.day, miles)            
            self.errorLabel.hide()

            self.milesEdit.setText('')
            self.showDetails()
            
        except ValueError:
            self.errorLabel.show()
            
            
        
        
class MainWindow(QtGui.QMainWindow):
    def __init__(self):
        QtGui.QMainWindow.__init__(self) 
        
        self.runningWidget = RunningWidget();
        
        
        self.setCentralWidget(self.runningWidget)

        #self.resize(500, 500)
        self.setWindowTitle('Running Log')
        
    def closeEvent (self, QCloseEvent):
        global sqlitedb         
        sqlitedb.shutdown()
        
        
app = QtGui.QApplication(sys.argv)

main = MainWindow()
main.show()

sys.exit(app.exec_())
